#!/usr/bin/php
<?
if ('cli' != php_sapi_name()) die();

define('DOKU_INC', '/usr/share/dokuwiki/');
require_once DOKU_INC.'inc/init.php';
require_once DOKU_INC.'inc/common.php';
require_once DOKU_INC.'inc/cliopts.php';

define('CACHE_FILE', 'titles.dat');
define('XSLT_PROCESSOR', 'xsltproc titles.xsl titles.xml');

/**
 * print message to stderr. sprintf format aware.
 */
function warn() {
	static $stderr = null;
	if ($stderr === null) {
		$stderr = fopen('php://stderr', 'w');
	}

	$args = func_get_args();
	$fmt = array_shift($args);
	fputs($stderr, vsprintf($fmt, $args));
}

/**
 * sprintf compatible die(), also sets exit code as 1
 */
function fatal() {
	$args = func_get_args();
	$fmt = array_shift($args);
	warn(vsprintf($fmt, $args));
	exit(1);
}

/**
 * get titles from external program
 */
function get_titles() {
	exec(XSLT_PROCESSOR, $lines, $rc);
	if ($rc) {
		fatal("XSLT processor failed: $rc");
	}

	$titles = array();
	foreach ($lines as $line) {
		list($id, $title) = explode('|', $line, 2);
		$titles[(int )$id] = $title;
	}
	return $titles;
}

function check_dups($titles) {
	$chk = array();
	$dups = array();
	foreach ($titles as $id => $title) {
		if (isset($chk[$title])) {
			$dups[$title] = $chk[$title] = array_merge($chk[$title], array($id));
		} else {
			$chk[$title] = array($id);
		}
	}

	if (count($dups)) {
		warn("Duplicates:\n");
		foreach ($dups as $title => $ids) {
			warn("- %s: %s\n", $title, join(', ', $ids));
		}
	}
}

function get_cache() {
	if (!file_exists(CACHE_FILE)) {
		$titles = get_titles();
		check_dups($titles);
		file_put_contents(CACHE_FILE, serialize(get_titles()));
	} else {
		$titles = unserialize(file_get_contents(CACHE_FILE));
	}
	return $titles;
}

$PROGRAM = basename($argv[0]);
if ($argc < 2) {
	fatal("Usage: $PROGRAM WIKI [NS] TITLE\n");
}
$cache = get_cache();
if ($argc < 3) {
	$ns = 'cowiki';
	$page = $argv[1];
	if (!isset($cache[$page])) {
		fatal("No such page: $page");
	}
	$title = $cache[$page];

} else {
	$ns = $argv[1];
	$title = $argv[2];
}

$fn = wikiFN($ns.':'.$title);
echo $fn;
